@@ -97,9 +97,10 @@ pkcomposite -i input1.tif -i input2.tif -o minimum.tif -cr minallbands
  | dy     | dy                   | double |       |Output resolution in y (in meter) (empty: keep original resolution) | 
  | e      | extent               | std::string |       |get boundary from extent from polygons in vector file | 
  | cut      | crop_to_cutline    | bool | false |Crop the extent of the target dataset to the extent of the cutline | 
+ | eo       | eo                 | std::string |       |special extent options controlling rasterization: ATTRIBUTE|CHUNKYSIZE|ALL_TOUCHED|BURN_VALUE_FROM|MERGE_ALG, e.g., -eo ATTRIBUTE=fieldname |
  | m      | mask                 | std::string |       |Use the first band of the specified file as a validity mask (0 is nodata) | 
  | msknodata | msknodata            | float | 0     |Mask value not to consider for composite
- | mskband | mskband              | short | 0     |Mask band to read (0 indexed). Provide band for each mask. | 
+ | mskband | mskband              | short | 0     |Mask band to read (0 indexed) | 
  | ulx    | ulx                  | double | 0     |Upper left x value bounding box | 
  | uly    | uly                  | double | 0     |Upper left y value bounding box | 
  | lrx    | lrx                  | double | 0     |Lower right x value bounding box | 
@@ -121,6 +122,8 @@ pkcomposite -i input1.tif -i input2.tif -o minimum.tif -cr minallbands
  | c      | class                | short | 0     |classes for multi-band output image: each band represents the number of observations for one specific class. Use value 0 for no multi-band output image. | 
  | ct     | ct                   | std::string |       |color table file with 5 columns: id R G B ALFA (0: transparent, 255: solid) | 
  | align  | align                | bool  |       |Align output bounding box to first input image | 
+ | scale  | scale                | double |       |output=scale*input+offset | 
+ | off    | offset               | double |       |output=scale*input+offset | 
  | d      | description          | std::string |       |Set image description | 
 
 Examples
@@ -147,9 +150,10 @@ int main(int argc, char *argv[])
   Optionpk<double>  dy_opt("dy", "dy", "Output resolution in y (in meter) (empty: keep original resolution)");
   Optionpk<string>  extent_opt("e", "extent", "get boundary from extent from polygons in vector file");
   Optionpk<bool> cut_opt("cut", "crop_to_cutline", "Crop the extent of the target dataset to the extent of the cutline.",false);
+  Optionpk<string> eoption_opt("eo","eo", "special extent options controlling rasterization: ATTRIBUTE|CHUNKYSIZE|ALL_TOUCHED|BURN_VALUE_FROM|MERGE_ALG, e.g., -eo ATTRIBUTE=fieldname");
   Optionpk<string> mask_opt("m", "mask", "Use the first band of the specified file as a validity mask (0 is nodata).");
   Optionpk<float> msknodata_opt("msknodata", "msknodata", "Mask value not to consider for composite.", 0);
-  Optionpk<short> mskband_opt("mskband", "mskband", "Mask band to read (0 indexed). Provide band for each mask.", 0);
+  Optionpk<short> mskband_opt("mskband", "mskband", "Mask band to read (0 indexed)", 0);
   Optionpk<double>  ulx_opt("ulx", "ulx", "Upper left x value bounding box", 0.0);
   Optionpk<double>  uly_opt("uly", "uly", "Upper left y value bounding box", 0.0);
   Optionpk<double>  lrx_opt("lrx", "lrx", "Lower right x value bounding box", 0.0);
@@ -172,10 +176,13 @@ int main(int argc, char *argv[])
   Optionpk<string>  colorTable_opt("ct", "ct", "color table file with 5 columns: id R G B ALFA (0: transparent, 255: solid)");
   Optionpk<string>  description_opt("d", "description", "Set image description");
   Optionpk<bool>  align_opt("align", "align", "Align output bounding box to input image",false);
+  Optionpk<double> scale_opt("scale", "scale", "output=scale*input+offset");
+  Optionpk<double> offset_opt("offset", "offset", "output=scale*input+offset");
   Optionpk<short>  verbose_opt("v", "verbose", "verbose", 0,2);
 
   extent_opt.setHide(1);
   cut_opt.setHide(1);
+  eoption_opt.setHide(1);
   mask_opt.setHide(1);
   msknodata_opt.setHide(1);
   mskband_opt.setHide(1);
@@ -185,6 +192,8 @@ int main(int argc, char *argv[])
   class_opt.setHide(1);
   colorTable_opt.setHide(1);
   description_opt.setHide(1);
+  scale_opt.setHide(1);
+  offset_opt.setHide(1);
 
   bool doProcess;//stop process when program was invoked with help option (-h --help)
   try{
@@ -195,6 +204,7 @@ int main(int argc, char *argv[])
     dy_opt.retrieveOption(argc,argv);
     extent_opt.retrieveOption(argc,argv);
     cut_opt.retrieveOption(argc,argv);
+    eoption_opt.retrieveOption(argc,argv);
     mask_opt.retrieveOption(argc,argv);
     msknodata_opt.retrieveOption(argc,argv);
     mskband_opt.retrieveOption(argc,argv);
@@ -220,6 +230,8 @@ int main(int argc, char *argv[])
     colorTable_opt.retrieveOption(argc,argv);
     description_opt.retrieveOption(argc,argv);
     align_opt.retrieveOption(argc,argv);
+    scale_opt.retrieveOption(argc,argv);
+    offset_opt.retrieveOption(argc,argv);
     verbose_opt.retrieveOption(argc,argv);
   }
   catch(string predefinedString){
@@ -269,6 +281,7 @@ int main(int argc, char *argv[])
     while(bndnodata_opt.size()<maxValue_opt.size())
       bndnodata_opt.push_back(bndnodata_opt[0]);
   }
+
   RESAMPLE theResample;
   if(resample_opt[0]=="near"){
     theResample=NEAR;
@@ -295,10 +308,10 @@ int main(int argc, char *argv[])
   vector<short> bands;
   
   //get bounding box
-  double maxLRX=0;
-  double maxULY=0;
-  double minULX=0;
-  double minLRY=0;
+  double maxLRX=lrx_opt[0];
+  double maxULY=uly_opt[0];
+  double minULX=ulx_opt[0];
+  double minLRY=lry_opt[0];
   double magic_x=1,magic_y=1;//magic pixel for GDAL map info
 
   GDALDataType theType=GDT_Unknown;
@@ -353,7 +366,15 @@ int main(int argc, char *argv[])
       }
       extentReader.close();
     }
-    if(cut_opt.size())
+    if(maxLRX>minULX&&minULX>ulx_opt[0])
+      ulx_opt[0]=minULX;
+    if(maxLRX>minULX&&maxLRX<lrx_opt[0])
+      lrx_opt[0]=maxLRX;
+    if(maxULY>minLRY&&maxULY<uly_opt[0])
+      uly_opt[0]=maxULY;
+    if(minLRY<maxULY&&minLRY>lry_opt[0])
+      lry_opt[0]=minLRY;
+    if(cut_opt.size()||eoption_opt.size())
       extentReader.open(extent_opt[0]);
   }
 
@@ -361,18 +382,23 @@ int main(int argc, char *argv[])
     cout << "--ulx=" << ulx_opt[0] << " --uly=" << uly_opt[0] << " --lrx=" << lrx_opt[0] << " --lry=" << lry_opt[0] << endl;
 
   vector<ImgReaderGdal> imgReader(input_opt.size());
+  // vector<ImgReaderGdal> imgReader(input_opt.size());
   string theProjection="";
   GDALColorTable* theColorTable=NULL;
   string imageType;
   bool init=false;
   for(int ifile=0;ifile<input_opt.size();++ifile){
     try{
-      imgReader[ifile].open(input_opt[ifile]);
+      imgReader[ifile].open(input_opt[ifile],GA_ReadOnly);
+      // imgReader[ifile].open(input_opt[ifile],GA_ReadOnly);
+      for(int iband=0;iband<scale_opt.size();++iband)
+        imgReader[ifile].setScale(scale_opt[iband],iband);
+      for(int iband=0;iband<offset_opt.size();++iband)
+        imgReader[ifile].setOffset(offset_opt[iband],iband);
     }
     catch(string errorstring){
       cerr << errorstring << " " << input_opt[ifile] << endl;
     }
-
     //todo: must be in init part only?
     if(colorTable_opt.empty())
       if(imgReader[ifile].getColorTable())
@@ -388,7 +414,6 @@ int main(int argc, char *argv[])
     if((ulx_opt[0]||uly_opt[0]||lrx_opt[0]||lry_opt[0])&&(!imgReader[ifile].covers(ulx_opt[0],uly_opt[0],lrx_opt[0],lry_opt[0]))){
       if(verbose_opt[0])
 	cout << input_opt[ifile] << " not within bounding box, skipping..." << endl;
-      // imgReader.close();
       continue;
     }
     double theULX, theULY, theLRX, theLRY;
@@ -470,7 +495,6 @@ int main(int argc, char *argv[])
       else
         imageType=imgReader[ifile].getImageType();
 
-      // dataType=imgReader.getDataType(0);
       if(verbose_opt[0]){
         cout << "type of data for " << input_opt[ifile] << ": " << theType << endl;
         cout << "nband: " << nband << endl;
@@ -488,19 +512,9 @@ int main(int argc, char *argv[])
 	dy=dy_opt[0];
       else
         dy=imgReader[ifile].getDeltaY();
-      // imgReader.getMagicPixel(magic_x,magic_y);
       init=true;
     }
     else{
-      //convert bounding box to magic coordinates
-      //check uniformity magic pixel
-      // double test_x,test_y;
-      // imgReader.getMagicPixel(test_x,test_y);
-      // if(verbose_opt[0]){
-      //   cout << "magic_x, magic_y: " << magic_x << ", " << magic_y << endl;
-      // }
-      // assert(magic_x==test_x);
-      // assert(magic_y==test_y);
       maxLRX=(theLRX>maxLRX)?theLRX:maxLRX;
       maxULY=(theULY>maxULY)?theULY:maxULY;
       minULX=(theULX<minULX)?theULX:minULX;
@@ -586,10 +600,10 @@ int main(int argc, char *argv[])
   }
   if(verbose_opt[0])
     cout << "open output image " << output_opt[0] << " with " << nwriteBand << " bands" << endl << flush;
+
   try{
     imgWriter.open(output_opt[0],ncol,nrow,nwriteBand,theType,imageType,option_opt);
-    for(int iband=0;iband<nwriteBand;++iband)
-      imgWriter.GDALSetNoDataValue(dstnodata_opt[0],iband);
+    imgWriter.setNoData(dstnodata_opt);
   }
   catch(string error){
     cout << error << endl;
@@ -625,7 +639,7 @@ int main(int argc, char *argv[])
   }
 
   ImgWriterGdal maskWriter;
-  if(extent_opt.size()&&cut_opt[0]){
+  if(extent_opt.size()&&(cut_opt[0]||eoption_opt.size())){
     try{
       maskWriter.open("/vsimem/mask.tif",ncol,nrow,1,GDT_Float32,"GTiff",option_opt);
       double gt[6];
@@ -643,10 +657,8 @@ int main(int argc, char *argv[])
 	  cout << "projection: " << theProjection << endl;
 	maskWriter.setProjection(theProjection);
       }
-	
-      //todo: handle multiple extent options
       vector<double> burnValues(1,1);//burn value is 1 (single band)
-      maskWriter.rasterizeOgr(extentReader,burnValues);
+      maskWriter.rasterizeOgr(extentReader,burnValues,eoption_opt);
       maskWriter.close();
     }
     catch(string error){
@@ -666,7 +678,7 @@ int main(int argc, char *argv[])
     try{
       if(verbose_opt[0]>=1)
 	std::cout << "opening mask image file " << mask_opt[0] << std::endl;
-      maskReader.open(mask_opt[0]);
+      maskReader.open(mask_opt[0],GA_ReadOnly);
       if(mskband_opt[0]>=maskReader.nrOfBand()){
 	string errorString="Error: illegal mask band";
 	throw(errorString);
@@ -689,7 +701,7 @@ int main(int argc, char *argv[])
   vector<short> fileBuffer(ncol);//holds the number of used files
   Vector2d<short> maxBuffer;//buffer used for maximum voting
   // Vector2d<double> readBuffer(nband);
-  vector<Vector2d<double> > readBuffer(input_opt.size());
+  vector<Vector2d<unsigned short> > readBuffer(input_opt.size());
   for(int ifile=0;ifile<input_opt.size();++ifile)
     readBuffer[ifile].resize(imgReader[ifile].nrOfBand());
   statfactory::StatFactory stat;
@@ -788,14 +800,20 @@ int main(int argc, char *argv[])
 	int readBand=(band_opt.size()>iband)? band_opt[iband] : iband;
         // readBuffer[iband].resize(readncol);
 	try{
-          imgReader[ifile].readData(readBuffer[ifile][iband],GDT_Float64,startCol,endCol,readRow,readBand,theResample);
+          imgReader[ifile].readData(readBuffer[ifile][iband],startCol,endCol,readRow,readBand,theResample);
+	  //test
+          // imgReader[ifile].readData(readBuffer[ifile][iband],static_cast<int>(startCol),static_cast<int>(endCol),static_cast<int>(readRow),static_cast<int>(readBand));
+	  // if(readRow==0&&iband==0){
+	  //   for(int icol=0;icol<10;++icol)
+	  //     cout << readBuffer[0][0][icol] << " ";
+	  //   cout << endl;
+	  // }
 	}
 	catch(string error){
 	  cerr << "error reading image " << input_opt[ifile] << ": " << endl;
 	  throw;
 	}
       }
-        
       for(int ib=0;ib<ncol;++ib){
         imgWriter.image2geo(ib,irow,x,y);
 	//check mask first
@@ -813,7 +831,7 @@ int main(int argc, char *argv[])
 
 	      assert(rowMask>=0&&rowMask<maskReader.nrOfRow());
 	      try{
-		maskReader.readData(lineMask,GDT_Float32,static_cast<int>(rowMask),mskband_opt[0]);
+		maskReader.readData(lineMask,static_cast<int>(rowMask),mskband_opt[0]);
 	      }
 	      catch(string errorstring){
 		cerr << errorstring << endl;
@@ -1197,7 +1215,7 @@ int main(int argc, char *argv[])
           for(int icol=0;icol<imgWriter.nrOfCol();++icol)
             classBuffer[icol]=maxBuffer[icol][class_opt[iclass]];
           try{
-            imgWriter.writeData(classBuffer,GDT_Int16,irow,iclass);
+            imgWriter.writeData(classBuffer,irow,iclass);
           }
           catch(string error){
             cerr << "error writing image file " << output_opt[0] << ": " << error << endl;
@@ -1214,9 +1232,9 @@ int main(int argc, char *argv[])
 	    fileBuffer[icol]=*(maxit);
         }
         try{
-          imgWriter.writeData(writeBuffer[0],GDT_Float64,irow,0);
+          imgWriter.writeData(writeBuffer[0],irow,0);
           if(file_opt[0])
-            imgWriter.writeData(fileBuffer,GDT_Int16,irow,1);
+            imgWriter.writeData(fileBuffer,irow,1);
         }
         catch(string error){
           cerr << "error writing image file " << output_opt[0] << ": " << error << endl;
@@ -1267,7 +1285,7 @@ int main(int argc, char *argv[])
 	  }
         }
         try{
-          imgWriter.writeData(writeBuffer[iband],GDT_Float64,irow,iband);
+          imgWriter.writeData(writeBuffer[iband],irow,iband);
         }
         catch(string error){
           cerr << error << " in " << output_opt[0] << endl;
@@ -1276,7 +1294,7 @@ int main(int argc, char *argv[])
       }
       if(file_opt[0]){
         try{
-          imgWriter.writeData(fileBuffer,GDT_Int16,irow,bands.size());
+          imgWriter.writeData(fileBuffer,irow,bands.size());
         }
         catch(string error){
           cerr << error << " in " << output_opt[0] << endl;
@@ -1287,11 +1305,12 @@ int main(int argc, char *argv[])
     progress=static_cast<float>(irow+1.0)/imgWriter.nrOfRow();
     pfnProgress(progress,pszMessage,pProgressArg);
   }
-  if(extent_opt.size()&&cut_opt.size()){
+  if(extent_opt.size()&&(cut_opt[0]||eoption_opt.size())){
     extentReader.close();
   }
-  for(int ifile=0;ifile<input_opt.size();++ifile)
+  for(int ifile=0;ifile<input_opt.size();++ifile){
     imgReader[ifile].close();
+  }
   if(mask_opt.size())
     maskReader.close();
   imgWriter.close();
