@@ -31,7 +31,6 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "fileclasses/FileReaderAscii.h"
 #include "imageclasses/ImgReaderGdal.h"
 #include "imageclasses/ImgWriterGdal.h"
-//test
 #include "algorithms/StatFactory.h"
 
 /******************************************************************************/
@@ -63,7 +62,7 @@ The number of output bands equals number of input bands
 
 |filter|description|
 |-------------|-------------|
-dilate|morphological dilation|
+|dilate|morphological dilation|
 |erode|morphological erosion|
 |close|morpholigical closing (dilate+erode)|
 |open|morpholigical opening (erode+dilate)|
@@ -170,6 +169,7 @@ order | rank pixels in order
 density | calculated the density
 homog | central pixel must be identical to all other pixels within window
 heterog | central pixel must be different than all other pixels within window
+sauvola | Sauvola's thresholding method
 
 Example: Sobel edge detection in horizontal direction
 
@@ -232,7 +232,7 @@ int main(int argc,char **argv) {
   // Optionpk<std::string> tmpdir_opt("tmp", "tmp", "Temporary directory","/tmp",2);
   Optionpk<bool> disc_opt("circ", "circular", "circular disc kernel for dilation and erosion", false);
   // Optionpk<double> angle_opt("a", "angle", "angle used for directional filtering in dilation (North=0, East=90, South=180, West=270).");
-  Optionpk<std::string> method_opt("f", "filter", "filter function (nvalid, median, var, min, max, sum, mean, dilate, erode, close, open, homog (central pixel must be identical to all other pixels within window), heterog (central pixel must be different than all other pixels within window), sobelx (horizontal edge detection), sobely (vertical edge detection), sobelxy (diagonal edge detection NE-SW),sobelyx (diagonal edge detection NW-SE), density, countid, mode (majority voting), only for classes), smoothnodata (smooth nodata values only) values, ismin, ismax, order (rank pixels in order), stdev, mrf, dwt, dwti, dwt_cut, dwt_cut_from, scramble, shift, savgolay, percentile, proportion)");
+  Optionpk<std::string> method_opt("f", "filter", "filter function (nvalid, median, var, min, max, sum, mean, dilate, erode, close, open, homog (central pixel must be identical to all other pixels within window), heterog (central pixel must be different than all other pixels within window), sauvola, sobelx (horizontal edge detection), sobely (vertical edge detection), sobelxy (diagonal edge detection NE-SW),sobelyx (diagonal edge detection NW-SE), density, countid, mode (majority voting), only for classes), smoothnodata (smooth nodata values only) values, ismin, ismax, order (rank pixels in order), stdev, mrf, dwt, dwti, dwt_cut, dwt_cut_from, scramble, shift, savgolay, percentile, proportion)");
   Optionpk<std::string> resample_opt("r", "resampling-method", "Resampling method for shifting operation (near: nearest neighbour, bilinear: bi-linear interpolation).", "near");
   Optionpk<double> dimX_opt("dx", "dx", "filter kernel size in x, use odd values only", 3);
   Optionpk<double> dimY_opt("dy", "dy", "filter kernel size in y, use odd values only", 3);
@@ -389,7 +389,9 @@ int main(int argc,char **argv) {
     theInterleave+=input.getInterleave();
     option_opt.push_back(theInterleave);
   }
+
   try{
+    string errorString;
     int nband=input.nrOfBand();
 
     if(fwhm_opt.size())
@@ -400,8 +402,8 @@ int main(int argc,char **argv) {
       nband=input.nrOfBand();
     else{
       if(method_opt.empty()){
-	cerr << "Error: no filter selected, use option -f" << endl;
-	exit(1);
+        errorString="Error: no filter selected, use option -f";
+        throw(errorString);
       }
       switch(filter2d::Filter2d::getFilterType(method_opt[0])){
       case(filter2d::dilate):
@@ -432,8 +434,8 @@ int main(int argc,char **argv) {
 	  assert(threshold_opt.size());
 	}
 	else{
-	  cerr << "filter not implemented in spatial domain" << endl;
-	  exit(1);
+          errorString="filter not implemented in spatial domain";
+          throw(errorString);
 	}
 	break;
       case(filter2d::mrf)://deliberate fall through
@@ -454,10 +456,11 @@ int main(int argc,char **argv) {
       case(filter2d::density):
       case(filter2d::homog):
       case(filter2d::heterog):
+      case(filter2d::sauvola):
 	//only implemented in spatial domain
 	if(dimZ_opt.size()){
-	  cerr << "filter not implemented in spectral/temporal domain" << endl;
-	  exit(1);
+          errorString="filter not implemented in spectral/temporal domain";
+          throw(errorString);
 	}
 	break;
       // case(filter2d::percentile):
@@ -487,12 +490,8 @@ int main(int argc,char **argv) {
 	  nband=input.nrOfBand();
 	break;
       default:
-	cerr << "filter not implemented" << endl;
-	exit(1);
-	// if(dimZ_opt.size())
-	//   nband=dimZ_opt[0];
-	// else
-	//   nband=input.nrOfBand();
+	errorString="filter not implemented";
+        throw(errorString);
 	break;
       }
     }
@@ -500,8 +499,8 @@ int main(int argc,char **argv) {
     output.open(output_opt[0],(input.nrOfCol()+down_opt[0]-1)/down_opt[0],(input.nrOfRow()+down_opt[0]-1)/down_opt[0],nband,theType,imageType,option_opt);
   }
   catch(string errorstring){
-    cout << errorstring << endl;
-    exit(4);
+    cerr << errorstring << endl;
+    exit(1);
   }
   output.setProjection(input.getProjection());
   double gt[6];
@@ -558,6 +557,9 @@ int main(int argc,char **argv) {
     if(verbose_opt[0])
       std::cout<< std::endl;
   }
+  filter1d.setThresholds(threshold_opt);
+  filter2d.setThresholds(threshold_opt);
+
   if(tap_opt.size()){
     ifstream tapfile(tap_opt[0].c_str());
     assert(tapfile);
@@ -613,11 +615,11 @@ int main(int argc,char **argv) {
       if((y+1+down_opt[0]/2)%down_opt[0])
         continue;
       for(int iband=0;iband<input.nrOfBand();++iband)
-        input.readData(lineInput[iband],GDT_Float64,y,iband);
+        input.readData(lineInput[iband],y,iband);
       filter1d.applyFwhm<double>(wavelengthIn_opt,lineInput,wavelengthOut_opt,fwhm_opt, interpolationType_opt[0], lineOutput, down_opt[0], verbose_opt[0]);
       for(int iband=0;iband<output.nrOfBand();++iband){
         try{
-          output.writeData(lineOutput[iband],GDT_Float64,y/down_opt[0],iband);
+          output.writeData(lineOutput[iband],y/down_opt[0],iband);
         }
         catch(string errorstring){
           cerr << errorstring << "in band " << iband << ", line " << y << endl;
@@ -668,7 +670,7 @@ int main(int argc,char **argv) {
       if((y+1+down_opt[0]/2)%down_opt[0])
         continue;
       for(int iband=0;iband<input.nrOfBand();++iband)
-        input.readData(lineInput[iband],GDT_Float64,y,iband);
+        input.readData(lineInput[iband],y,iband);
       for(int isrf=0;isrf<srf.size();++isrf){
         vector<double> lineOutput(output.nrOfCol());
         double delta=1.0;
@@ -820,6 +822,21 @@ int main(int argc,char **argv) {
       }
       break;
     }
+    case(filter2d::sauvola):{//Implements Sauvola's thresholding method (http://fiji.sc/Auto_Local_Threshold)
+
+  //test
+      Vector2d<unsigned short> inBuffer;
+      for(int iband=0;iband<input.nrOfBand();++iband){
+        input.readDataBlock(inBuffer,0,input.nrOfCol()-1,0,input.nrOfRow()-1,iband);
+      }
+      try{
+	filter2d.doit(input,output,"sauvola",dimX_opt[0],dimY_opt[0],down_opt[0],disc_opt[0]);
+      }
+      catch(string errorstring){
+	cerr << errorstring << endl;
+      }
+      break;
+    }
     case(filter2d::shift):{//shift
       if(down_opt[0]!=1){
 	std::cerr << "Error: down option not supported for shift operator" << std::endl;
