@@ -32,7 +32,7 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 ## SYNOPSIS
 
 <code>
-  
+  Usage: pklas2img -i input.las -o output.tif
 </code>
 
 <code>
@@ -45,17 +45,17 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 
 \section pklas2img_description Description
 
-The utility pklas2img converts a las/laz point cloud into a gridded raster dataset. The implementation is based on <a href="www.liblas.org">liblas</a> API. You can define the bounding box, grid cell size and spatial reference set. The composite rule for multiple returns within a single grid cell can be set with the option -comp. The default attribute is z (heiht), but can also be intensity (if available), scan angle rank (-n angle), the return number (-n return) or the total number of returns in that grid cell (-n nreturn). To select specific returns only, set the option -fir (first, last, single, multiple, or all).
+The utility pklas2img converts a las/laz point cloud into a gridded raster dataset. The implementation is based on <a href="http://www.liblas.org">liblas</a> API. You can define the bounding box, grid cell size and spatial reference set. The composite rule for multiple returns within a single grid cell can be set with the option -comp. The default attribute is z (heiht), but can also be intensity (if available), scan angle rank (-n angle), the return number (-n return) or the total number of returns in that grid cell (-n nreturn). To select specific returns only, set the option -fir (first, last, single, multiple, or all).
 \section pklas2img_options Options
  - use either `-short` or `--long` options (both `--long=value` and `--long value` are supported)
  - short option `-h` shows basic options only, long option `--help` shows all options
 |short|long|type|default|description|
 |-----|----|----|-------|-----------|
  | i      | input                | std::string |       |Input las file | 
- | n      | name                 | std::string | z     |names of the attribute to select: intensity, angle, return, nreturn, z | 
+ | n      | name                 | std::string | z     |names of the attribute to select: intensity, angle, return, nreturn, spacing, z | 
  | ret    | ret                  | unsigned short |       |number(s) of returns to include | 
  | class  | class                | unsigned short |       |classes to keep: 0 (created, never classified), 1 (unclassified), 2 (ground), 3 (low vegetation), 4 (medium vegetation), 5 (high vegetation), 6 (building), 7 (low point, noise), 8 (model key-point), 9 (water), 10 (reserved), 11 (reserved), 12 (overlap) | 
- | comp   | comp                 | std::string | last  |composite for multiple points in cell (min, max, median, mean, sum, first, last, profile (percentile height values), percentile, number (point density)). Last: overwrite cells with latest point | 
+ | comp   | comp                 | std::string | last  |composite for multiple points in cell (min, max, absmin, absmax, median, mean, sum, first, last, profile (percentile height values), percentile, number (point density)). Last: overwrite cells with latest point | 
  | fir    | filter               | std::string | all   |filter las points (first,last,single,multiple,all). | 
  | angle_min  | angle_min        | unsigned short |    |minimum scan angle to read points. | 
  | angle_max  | angle_max        | unsigned short |    |maximum scan angle to read points. | 
@@ -84,17 +84,17 @@ using namespace std;
 
 int main(int argc,char **argv) {
   Optionpk<string> input_opt("i", "input", "Input las file");
-  Optionpk<string> attribute_opt("n", "name", "names of the point attribute to select: intensity, angle, return, nreturn, angle, z", "z");
+  Optionpk<string> attribute_opt("n", "name", "names of the point attribute to select: intensity, angle, return, nreturn, spacing, z", "z");
   // Optionpk<bool> disc_opt("circ", "circular", "circular disc kernel for dilation and erosion", false);
   // Optionpk<double> maxSlope_opt("s", "maxSlope", "Maximum slope used for morphological filtering", 0.0);
   // Optionpk<double> hThreshold_opt("ht", "maxHeight", "initial and maximum height threshold for progressive morphological filtering (e.g., -ht 0.2 -ht 2.5)", 0.2);
   // Optionpk<short> maxIter_opt("maxit", "maxit", "Maximum number of iterations in post filter", 5);
   Optionpk<unsigned short> returns_opt("ret", "ret", "number(s) of returns to include");
   Optionpk<unsigned short> classes_opt("class", "class", "classes to keep: 0 (created, never classified), 1 (unclassified), 2 (ground), 3 (low vegetation), 4 (medium vegetation), 5 (high vegetation), 6 (building), 7 (low point, noise), 8 (model key-point), 9 (water), 10 (reserved), 11 (reserved), 12 (overlap)");
-  Optionpk<string> composite_opt("comp", "comp", "composite for multiple points in cell (min, max, median, mean, sum, first, last, profile (percentile height values), percentile, number (point density)). Last: overwrite cells with latest point", "last");
+  Optionpk<string> composite_opt("comp", "comp", "composite for multiple points in cell (min, max, absmin, absmax, median, mean, sum, first, last, profile (percentile height values), percentile, number (point density)). Last: overwrite cells with latest point", "last");
   Optionpk<string> filter_opt("fir", "filter", "filter las points (first,last,single,multiple,all).", "all");
-  Optionpk<unsigned short> angle_min_opt("angle_min", "angle_min", "Minimum scan angle to read points.");
-  Optionpk<unsigned short> angle_max_opt("angle_max", "angle_max", "Maximum scan angle to read points.");
+  Optionpk<short> angle_min_opt("angle_min", "angle_min", "Minimum scan angle to read points.");
+  Optionpk<short> angle_max_opt("angle_max", "angle_max", "Maximum scan angle to read points.");
   // Optionpk<string> postFilter_opt("pf", "pfilter", "post processing filter (etew_min,promorph (progressive morphological filter),bunting (adapted promorph),open,close,none).", "none");
   // Optionpk<short> dimx_opt("dimx", "dimx", "Dimension X of postFilter", 3);
   // Optionpk<short> dimy_opt("dimy", "dimy", "Dimension Y of postFilter", 3);
@@ -185,6 +185,10 @@ int main(int argc,char **argv) {
                  otype_opt[0].c_str()))
       theType=(GDALDataType) iType;
   }
+  if(attribute_opt[0]=="spacing"){
+    if(theType!=GDT_Float32||theType!=GDT_Float64)
+      theType=GDT_Float32;
+  }
   if(verbose_opt[0]){
     if(theType==GDT_Unknown)
       cout << "Unknown output pixel type: " << otype_opt[0] << endl;
@@ -334,7 +338,7 @@ int main(int argc,char **argv) {
             std::cout << "writing intensity" << std::endl;
           ++ait;
         }
-        if(*ait=="angle"){
+        else if(*ait=="angle"){
           if(verbose_opt[0])
             std::cout << "writing angle" << std::endl;
           ++ait;
@@ -349,17 +353,35 @@ int main(int argc,char **argv) {
             std::cout << "writing number of returns" << std::endl;
           ++ait;
         }
+        else if(*ait=="spacing"){
+          if(verbose_opt[0])
+            std::cout << "writing spacing" << std::endl;
+          ++ait;
+        }
         else
           attribute_opt.erase(ait);
       }
     }
-    liblas::Point thePoint(&(lasReader.getHeader()));
-    while(lasReader.readNextPoint(thePoint)){
+
+    // liblas::Point thePoint(&(lasReader.getHeader()));
+    // while(lasReader.readNextPoint(thePoint)){
+    OGRSpatialReference projectionRef(outputWriter.getProjectionRef().c_str());
+    OGRPoint ogrPoint;
+    OGRPoint ogrCenter;
+    if(attribute_opt[0]=="spacing"){
+      ogrPoint.assignSpatialReference(&projectionRef);
+      ogrCenter.assignSpatialReference(&projectionRef);
+    }
+    while(lasReader.getReader()->ReadNextPoint()){
+      liblas::Point const& thePoint = lasReader.getReader()->GetPoint();
+      // liblas::Point const& thePoint=lasReader.getPoint();
       progress=static_cast<float>(ipoint)/totalPoints;
       pfnProgress(progress,pszMessage,pProgressArg);
+      double theX=thePoint.GetX();
+      double theY=thePoint.GetY();
       if(verbose_opt[0]>1)
         cout << "reading point " << ipoint << endl;
-      if(thePoint.GetX()<minULX||thePoint.GetX()>=maxLRX||thePoint.GetY()>=maxULY||thePoint.GetY()<minLRY)
+      if(theX<minULX||theX>=maxLRX||theY>=maxULY||theY<minLRY)
         continue;
       if((filter_opt[0]=="single")&&(thePoint.GetNumberOfReturns()!=1))
         continue;
@@ -378,7 +400,7 @@ int main(int argc,char **argv) {
 	  continue;
       }
       double dcol,drow;
-      outputWriter.geo2image(thePoint.GetX(),thePoint.GetY(),dcol,drow);
+      outputWriter.geo2image(theX,theY,dcol,drow);
       int icol=static_cast<int>(dcol);
       int irow=static_cast<int>(drow);
       if(irow<0||irow>=nrow){
@@ -407,6 +429,16 @@ int main(int argc,char **argv) {
         inputData[irow][icol].push_back(thePoint.GetReturnNumber());
       else if(attribute_opt[0]=="nreturn")
         inputData[irow][icol].push_back(thePoint.GetNumberOfReturns());
+      else if(attribute_opt[0]=="spacing"){
+	ogrPoint.setX(theX);
+	ogrPoint.setY(theY);
+	double centerX;
+	double centerY;
+	outputWriter.image2geo(icol,irow,centerX,centerY);
+	ogrCenter.setX(centerX);
+	ogrCenter.setY(centerY);
+	inputData[irow][icol].push_back(ogrPoint.Distance(&ogrCenter));
+      }
       else{
         std::string errorString="attribute not supported";
         throw(errorString);
@@ -444,6 +476,10 @@ int main(int argc,char **argv) {
           outputData[irow][icol]=stat.mymin(inputData[irow][icol]);
         else if(composite_opt[0]=="max")
           outputData[irow][icol]=stat.mymax(inputData[irow][icol]);
+        else if(composite_opt[0]=="absmin")
+          outputData[irow][icol]=stat.absmin(inputData[irow][icol]);
+        else if(composite_opt[0]=="absmax")
+          outputData[irow][icol]=stat.absmax(inputData[irow][icol]);
         else if(composite_opt[0]=="median")
           outputData[irow][icol]=stat.median(inputData[irow][icol]);
         else if(composite_opt[0]=="percentile")
@@ -489,7 +525,7 @@ int main(int argc,char **argv) {
         // assert(outputProfile[iband].size()==outputWriter.nrOfRow());
         assert(outputProfile[iband].size()==outputWriter.nrOfCol());
         try{
-          outputWriter.writeData(outputProfile[iband],GDT_Float64,irow,iband);
+          outputWriter.writeData(outputProfile[iband],irow,iband);
         }
         catch(std::string errorString){
           cout << errorString << endl;
@@ -609,7 +645,7 @@ int main(int argc,char **argv) {
       try{
         assert(outputData.size()==outputWriter.nrOfRow());
         assert(outputData[0].size()==outputWriter.nrOfCol());
-        outputWriter.writeData(outputData[irow],GDT_Float64,irow,0);
+        outputWriter.writeData(outputData[irow],irow,0);
       }
       catch(std::string errorString){
         cout << errorString << endl;
